/*-----------------------------------------------------------------------

File  : che_proofcontrol.c

Author: Stephan Schulz

Contents
 
  Basic functions for proof control objects.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Oct 16 16:20:00 MET DST 1998
    Salvaged from cco_proofstate.h, forked control and state.
<2> Wed Dec 16 18:45:14 MET 1998
    Moved from cco to che

-----------------------------------------------------------------------*/

#include "che_proofcontrol.h"


/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

char* DefaultWeightFunctions = 
"\n"
"weight21_ugg  = Clauseweight(PreferUnitGroundGoals,2,1,1)      \n"
"rweight21_g   = Refinedweight(PreferGoals,    2,1,1.5,1.1,1) \n"
"rweight11_g   = Refinedweight(PreferGoals,    1,1,1.5,1.1,1.1) \n"
"rweight21_a   = Refinedweight(PreferNonGoals, 2,1,1.5,1.1,1.1) \n"
"rweight21_ugg = Refinedweight(PreferUnitGroundGoals, 2,1,1.5,1.1,1.1) \n"
"fifo_f        = FIFOWeight(ConstPrio)                      \n"
"lifo_f        = LIFOWeight(ConstPrio)                      \n"
"weight11_f    = Clauseweight(ConstPrio,1,1,1)              \n"
"weight21_f    = Clauseweight(ConstPrio,2,1,1)              \n"
;

char* DefaultHeuristics = 
"Weight     = (1*weight21_ugg)                       \n"
"RWeight    = (1*rweight21_ugg)                      \n"
"FIFO       = (1*fifo_f)                             \n"
"LIFO       = (1*lifo_f)                             \n"
"Default    = (3*rweight21_a, 1*rweight21_g)         \n"
"Uniq       = (1*Uniqweight(ConstPrio))\n"
"/* Here start the auto-mode-heuristics*/            \n"

"/* H*-Strategies: */  \n"

"H_____L1o = \n"
"    (8*Refinedweight(PreferGoals,1,2,2,2,1),\n"
"      8*Refinedweight(PreferNonGoals,2,1,2,2,1),\n"
"      1*Clauseweight(ConstPrio,1,1,1),\n"
"      1*FIFOWeight(ConstPrio))\n"
"H____Aa = \n"
"    (10*Refinedweight(PreferGoals,1,2,2,2,0.5),\n"
"      10*Refinedweight(PreferNonGoals,2,1,2,2,2),\n"
"      1*Clauseweight(ConstPrio,1,1,1),\n"
"      1*FIFOWeight(ConstPrio))\n"
"H____Ba = \n"
"    (8*Refinedweight(PreferGoals,1,2,2,2,2),\n"
"      8*Refinedweight(PreferNonGoals,2,1,2,2,0.5),\n"
"      1*Clauseweight(PreferUnitGroundGoals,1,1,1),\n"
"      1*FIFOWeight(ConstPrio))\n"
"H_____LFj = \n"
"    (8*Refinedweight(ConstPrio,1,1,2,2,1),\n"
"     1*FIFOWeight(ConstPrio))\n"
"H_____L0h = \n"
"    (10*Refinedweight(PreferGroundGoals,2,1,2,1.5,1),\n"
"     1*Clauseweight(ConstPrio,1,1,1),\n"
"     1*FIFOWeight(ConstPrio))\n"
"H____7a = \n"
"    (1*Refinedweight(PreferGoals,1,2,1.5,1,1),\n"
"      2*Refinedweight(PreferNonGoals,2,1,1.5,1,1))\n"
"H____Cc = \n"
"    (10*Sigweight(PreferGoals,1,2,2,2,1,-0.3),\n"
"      10*Sigweight(PreferNonGoals,2,1,2,2,1,-0.3),\n"
"      1*Clauseweight(ConstPrio,1,1,1),\n"
"      1*FIFOWeight(ConstPrio))\n"
"H____7c = \n"
"    (1*Refinedweight(PreferGoals,1,2,1.5,1,1),\n"
"      2*Refinedweight(PreferNonGoals,2,1,1.5,1,1))\n"
"H____Eb = \n"
"    (8*Refinedweight(PreferGoals,1,2,2,2,1),\n"
"      8*Refinedweight(PreferNonGoals,2,1,2,2,1),\n"
"      1*Clauseweight(ConstPrio,1,1,1),\n"
"      1*FIFOWeight(ByNegLitDist))\n"
"H_____L1j = \n"
"    (8*Refinedweight(PreferGoals,1,2,2,2,1),\n"
"      8*Refinedweight(PreferNonGoals,2,1,2,2,1),\n"
"      1*Clauseweight(ConstPrio,1,1,1),\n"
"      1*FIFOWeight(ConstPrio))\n"
"H_____L3h = \n"
"    (10*Sigweight(PreferGoals,1,2,2,2,1,-0.3),\n"
"      10*Sigweight(PreferNonGoals,2,1,2,2,1,-0.3),\n"
"      1*Clauseweight(ConstPrio,1,1,1),\n"
"      1*FIFOWeight(ConstPrio))\n"

"/* G* Strategies: */  \n"

"G____N14h = \n"
"    (3*Refinedweight(PreferGoals,1,1,2,1.5,2),\n"
"     3*Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),\n"
"     1*Clauseweight(ConstPrio,1,1,1.5),\n"
"     1*FIFOWeight(ConstPrio))\n"
"G____N14j = \n"
"    (3*Refinedweight(PreferGoals,1,1,2,1.5,2),\n"
"     3*Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),\n"
"     1*Clauseweight(ConstPrio,1,1,1.5),\n"
"     1*FIFOWeight(ConstPrio))\n"
"G_____R = \n"
"    (8*Refinedweight(ConstPrio,1,1,2,1.4,1),\n"
"     1*FIFOWeight(ConstPrio))\n"
"G_____Vn = \n"
"    (3*Refinedweight(PreferGoals,1,1,2,1.2,2),\n"
"     3*Refinedweight(PreferNonGoals,1,1,2,1.2,0.5),\n"
"     1*Clauseweight(PreferUnitGroundGoals,1,1,1),\n"
"     1*FIFOWeight(ConstPrio))\n"
"G____N14f = \n"
"    (3*Refinedweight(PreferGoals,1,1,2,1.5,2),\n"
"     3*Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),\n"
"     1*Clauseweight(ConstPrio,1,1,1.5),\n"
"     1*FIFOWeight(ConstPrio))\n"
"G____N05j = \n"
"    (5*Refinedweight(ConstPrio,2,1,2,1,1),\n"
"     1*FIFOWeight(ConstPrio))\n"
"G_____Z = \n"
"    (3*Refinedweight(PreferGoals,1,1,2,1.5,2),\n"
"     3*Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),\n"
"     1*Clauseweight(ConstPrio,1,1,1.5),\n"
"     1*FIFOWeight(PreferUnitGroundGoals))\n"
"G____N14g = \n"
"    (3*Refinedweight(PreferGoals,1,1,2,1.5,2),\n"
"     3*Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),\n"
"     1*Clauseweight(ConstPrio,1,1,1.5),\n"
"     1*FIFOWeight(ConstPrio))\n"

"/* UU*G Strategies */ \n"

"U___GJ = \n"
"    (25*Sigweight(PreferGoals,2,1,3.0,1,1,-0.3),\n"
"     5*Clauseweight(ConstPrio,1,1,1),\n"
"     1*FIFOWeight(ConstPrio))\n"
"U___G_12 = \n"
"    (12*Refinedweight(PreferGoals,2,1,3.5,1,1),\n"
"     1*ClauseLMaxWeight(ConstPrio,2,1,1),\n"
"     1*FIFOWeight(ConstPrio))\n"

"/* UU*N Strategies */ \n"

"U___NU = \n"
"    (1*Clauseweight(ConstPrio,1,1,2))\n"
"U___N_I = \n"
"    (1*Refinedweight(PreferNonGoals,1,2,2,1,1),\n"
"     4*Clauseweight(PreferGoals,1,1,1))\n"

"/* UH* Strategies  */ \n"

"UH____Ka = \n"
"    (2*Refinedweight(PreferGoals,1,2,2,2,1),\n"
"     5*Refinedweight(ByGoalDifficulty,1,2,2,2,1),\n"
"      10*Refinedweight(PreferNonGoals,2,1,2,2,1))\n"
"UH____Yn = \n"
"    (1*ClauseLMaxWeight(PreferGoals,1,1,1),\n"
"     1*ClauseLMaxWeight(PreferNonGoals,2,1,1))\n"

;


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: ProofControlFree()
//
//   Allocate an empty, initialized ProofControlCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

ProofControl_p ProofControlAlloc()
{
   ProofControl_p handle = ProofControlCellAlloc();
   handle->ocb                     = NULL;
   handle->wfcbs                   = WFCBAdminAlloc();
   handle->hcbs                    = HCBAdminAlloc();
   handle->hcb                     = NULL;
   handle->prefer_initial_clauses  = false;
   handle->selection_strategy      = SelectNoLiterals;
   handle->pos_lit_sel_min         = 0; 
   handle->pos_lit_sel_max         = LONG_MAX; 
   handle->neg_lit_sel_min         = 0; 
   handle->neg_lit_sel_max         = LONG_MAX; 
   handle->all_lit_sel_min         = 0; 
   handle->all_lit_sel_max         = LONG_MAX; 
   return handle;
}

/*-----------------------------------------------------------------------
//
// Function: ProofControlFree()
//
//   Free a ProofControlCell.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void ProofControlFree(ProofControl_p junk)
{
   assert(junk);
   if(junk->ocb)
   {
      OCBFree(junk->ocb);
   }
   WFCBAdminFree(junk->wfcbs);
   HCBAdminFree(junk->hcbs);
   /* hcb is always freed in junk->hcbs */
   ProofControlCellFree(junk);
}


/*-----------------------------------------------------------------------
//
// Function: DoLiteralSelection()
//
//   Based on control, select a literal selection strategy and apply
//   it to clause.
//
// Global Variables: -
//
// Side Effects    : Changes properties in clause
//
/----------------------------------------------------------------------*/

void DoLiteralSelection(ProofControl_p control, Clause_p clause)
{
   if((clause->pos_lit_no >= control->pos_lit_sel_min) && 
      (clause->pos_lit_no <= control->pos_lit_sel_max) &&
      (clause->neg_lit_no >= control->neg_lit_sel_min) && 
      (clause->neg_lit_no <= control->neg_lit_sel_max) &&
      (ClauseLiteralNumber(clause) >= control->all_lit_sel_min) &&
      (ClauseLiteralNumber(clause) <= control->all_lit_sel_max))
   {
      control->selection_strategy(clause);
   }
   else
   {
      SelectNoLiterals(clause); 
   }
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/







