/*-----------------------------------------------------------------------

File  : che_to_precgen.h

Author: Stephan Schulz

Contents
 
  Routines for generating precedences for term orderings

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Oct 16 17:01:23 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CHE_TO_PRECGEN

#define CHE_TO_PRECGEN

#include <clb_simple_stuff.h>
#include <che_clausesetfeatures.h>
#include <che_lpogen.h>
#include <stdlib.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum
{
   PNoMethod = 0,       /* Nothing */
   PUnaryFirst,         /* My hack ;-) */
   PArity,              /* Sort by arity */
   PInvArity,           /* Sort by inverse arity */
   PConstMax,           /* Sort by arity, but constants first (Default
			   for SPASS) */
   PByFrequency,        /* Make often occuring symbols big */
   PByInvFrequency,     /* Make often occuring symbols small */
   PWeightAxioms,       /* Joachims hack */
   PMinMethod = PUnaryFirst,
   PMaxMethod = PWeightAxioms
}TOPrecGenMethod;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


extern char* TOPrecGenNames[];

#define TOGetPrecGenName(method) \
        (assert((method)>=PNoMethod), assert((method)<=PMaxMethod), \
	 TOPrecGenNames[(method)])

TOPrecGenMethod TOTranslatePrecGenMethod(char* name);

#define TOGenerateDefaultPrecedence(ocb) \
        TOGeneratePrecedence((ocb), NULL, PUnaryFirst)

void TOGeneratePrecedence(OCB_p ocb, ClauseSet_p axioms,
			  TOPrecGenMethod method); 


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/






