#!/usr/local/dist/bin/gawk -f

# Usage: generate_auto.awk [<file>]
#
#   Given the output of find_optimal.awk, generate a selection of
#   heuristics to use (and possibly print out the C code to realize
#   this selection).
#
# Copyright 1999 Stephan Schulz, schulz@informatik.tu-muenchen.de

function is_empty(array)
{
   for(i in array)
   {
      return false;
   }
   return true;
}

function get_preorder(    pipe, tmp,  name, sols, time)
{
   i=0;

   pipe = "geauswert.sh";
   while((pipe | getline tmp)>0)
   {      
      name = substr(tmp, 11, index(tmp, " ")-11);
      match(tmp, /B: [0-9]*/);
      sols = substr(tmp, RSTART+3, RLENGTH);
      match(tmp, /e: [0-9]*\.[0-9]*/);
      time = substr(tmp, RSTART+3, RLENGTH);
      global_strat_preord[name] = sprintf("%4d %6.11f", sols, 100000-time);
   }
   close(pipe);
}

function find_best_global(   i, init, res)
{
   init = "   ";
   res = "";

   for(i in global_strat_preord)
   {
      if(global_strat_preord[i] > init)
      {
	 init = global_strat_preord[i];
	 res = i;
      }
   }
   return res;
}



function max(a,b)
{
  if(a>b)
    {
      return a;
    }
  return b;
}

function remove_strat(class, strat)
{
   #print "Strat: " strat;
   #print strategies[class];
   if(index(strategies[class], strat "|"))
   {
      #print "Remove " class;
      strat_classes[strat] = strat_classes[strat] class "|";
      delete strategies[class];
      return 1;
   }
   return 0;
}

function global_remove_strat(strat,     class, res)
{
   res = 0;
   for(class in strategies)
   {
      #print "gr: ", class, strat;
      res+=remove_strat(class, strat);
   }
   #print "Removed " res " classes for " strat;
   return res;
}


# For the best class with sol_no solutions find an appropriate
# strategy.

function pick_best_strategy(sol_no,     i, tmp, tmp1, pos)
{
   strat_eval[0] = "";
   delete strat_eval;
   
   for(i in strategies)
   {
      if(solutions[i] == sol_no)
      {
	 tmp = strategies[i];
	 while(tmp)
	 {
	    pos = index(tmp, "|");
	    tmp1=substr(tmp, 1, pos-1);
	    strat_eval[tmp1]++;
	    tmp = substr(tmp, pos+1);
	 }
      }
   }
   tmp = 0;
   tmp1 = "";
   for(i in strat_eval)
   {
      if((strat_eval[i]>tmp) || 
	 ((strat_eval[i]==tmp) && 
	  (global_strat_preord[i]>global_strat_preord[tmp1])))
      {
	 tmp = strat_eval[i];
	 tmp1 = i;
      }
   }
   if(tmp1)
   {
      total_removed += global_remove_strat(tmp1);
   }
   return tmp1;
}

function generate_single_cond(array, letter,             res)
{
   res = "";
   if(array[letter])
   {
      res = global_prefix "Spec" array[letter] "(spec)";
      global_prefix = "&&\n       ";
   }
   return res;
}

function translate_class_to_cond(class,  res)
{
   res = "      ( /* " class " */\n";

   global_prefix = "       ";
   res = res generate_single_cond(ax_int,substr(class, 1, 1));
   res = res generate_single_cond(go_int,substr(class, 2, 1));
   res = res generate_single_cond(eq_int,substr(class, 3, 1));
   res = res generate_single_cond(ng_int,substr(class, 4, 1));
   res = res generate_single_cond(gg_int,substr(class, 5, 1));
   res = res generate_single_cond(an_int,substr(class, 6, 1));
   res = res generate_single_cond(lt_int,substr(class, 7, 1));
   res = res generate_single_cond(ts_int,substr(class, 8, 1));
   res = res ")";

   return res;
}

function translate_class_list_to_cond(list,  res, sep, found, tmp)
{
   res = "(\n";
   sep = "";
   tmp = list;
   
   while((found = index(tmp, "|")))
   {
      res = res sep translate_class_to_cond(substr(tmp, 1, found-1));
      sep = "\n        ||\n";
      tmp = substr(tmp, found+1);
   }   
   res = res ")";

   return res;
}


BEGIN{
   class = "";
   
   ax_int["U"] = "AxiomsAreUnit";
   ax_int["H"] = "AxiomsAreHorn";
   ax_int["G"] = "AxiomsAreGeneral";
   go_int["U"] = "GoalsAreUnit";
   go_int["H"] = "GoalsAreHorn";
   go_int["G"] = "GoalsAreGeneral";
   eq_int["N"] = "NoEq";
   eq_int["S"] = "SomeEq";
   eq_int["P"] = "PureEq";
   ng_int["F"] = "FewNGPosUnits";
   ng_int["S"] = "SomeNGPosUnits";
   ng_int["M"] = "ManyNGPosUnits";
   gg_int["N"] = "GoalsHaveVars";
   gg_int["G"] = "GoalsAreGround";
   an_int["F"] = "FewAxioms";
   an_int["S"] = "SomeAxioms";
   an_int["M"] = "ManyAxioms";
   lt_int["F"] = "FewLiterals";
   lt_int["S"] = "SomeLiterals";
   lt_int["M"] = "ManyLiterals";
   ts_int["S"] = "SmallTerms";
   ts_int["M"] = "MediumTerms";
   ts_int["L"] = "LargeTerms";

   strat["a"] = "SelectNoLiterals";
   strat["b"] = "SelectNegativeLiterals";          
   strat["c"] = "SelectPosNegativeLiterals";       
   strat["d"] = "SelectFirstVariableLiteral";      
   strat["e"] = "SelectPosFirstVariableLiteral";   
   strat["f"] = "SelectLargestNegativeLiteral";    
   strat["g"] = "SelectPosLargestNegativeLiteral"; 
   strat["h"] = "SelectSmallestNegativeLiteral";   
   strat["i"] = "SelectPosSmallestNegativeLiteral";
   strat["j"] = "SelectDiffNegativeLiteral";       
   strat["k"] = "SelectPosDiffNegativeLiteral";    
   strat["l"] = "SelectGroundNegativeLiteral";     
   strat["m"] = "SelectPosGroundNegativeLiteral";  
   strat["n"] = "SelectOptimalLiteral";            
   strat["o"] = "SelectPosOptimalLiteral";         

   get_preorder();
}
   
   

/\.\./{
   match($0, /\([0-9]+\//);
   sols = 0+substr($0, RSTART+1, RLENGTH-1);
   match($0, /\/[0-9]+\)/);
   size = 0+substr($0, RSTART+1, RLENGTH-1);   
   match($0, /\.\.\/[^\/]*\//);
   class = substr($0, RSTART+RLENGTH);
   match(class, /CLASS_[^ ]* /);
   class = substr(class, RSTART+6,RLENGTH-7);
   solutions[class] = 0;
   strategies[class] = "";
   successes[class] = sols;
   class_size[class] = size;
}

/  protokoll_/{
   solutions[class]++;
   match($0, /protokoll_.*:/);
   strategy = substr($0, RSTART+10, RLENGTH-11);
   strategies[class] = strategies[class] strategy "|";
   strat_list[strategy] += sols;
}


function print_strategy(to_print)
{
   print "   {";
   print "      res = \"" gensub(/-/, "_", "g", to_print)  "\";";
   
   selstrat = substr(to_print, length(to_print), 1);
   if(selstrat in strat)
   {
      print "      control->selection_strategy = " strat[selstrat] ";";	  
   }
   else
   {
      print "      control->selection_strategy = " strat["a"] ";";	
   }
   
   print "   }";
}

function print_definition(to_define,   data, def, newdef)
{
   pipe =  "grep \"H'(.*)\" protokoll_" to_define; 
   pipe | getline data;
   close(pipe);	 
   match(data, /-H'\(.*\)'/);
   current_class = gensub(/-/, "_", "g", to_define);
   def = substr(data, RSTART+3, RLENGTH-4);
   
   newdef = gensub(/\) *\,/, "),\\\\n\"\n\"     ", "g", def);
   
   print "\"" current_class " = \\n\"\n\"    " newdef "\\n\"";
}


END{
   max_solutions = 0;
   for(i in solutions)
   {
      max_solutions = max(max_solutions, solutions[i]);
   }
   
   total_removed = 0;
   for(i=1; i <= max_solutions; i++)
   {
      while(pick_best_strategy(i)); 
   }

   print "   /* --------------------------------------------------------";
   print "       The following code is generated automatically with";
   print "       generate_auto.awk. Yes, it is fairly ugly ;-)";
   print "";
   print "       Classes used (Copy this into che_proofcontrol.c):\n";

   for(i in strat_classes)
   {
      print_definition(i);
   }
   global_best = find_best_global();
   if(global_best in strat_classes)
   {
      print "    Global best, " global_best ", already defined";
   }
   else
   {
      print_definition(global_best);
   }
   print "    ----------------------------------------------------------*/";

   for(i in strat_classes)
   {
      print "   else if" translate_class_list_to_cond(strat_classes[i]);
      print_strategy(i);
   }
   print "   else /* Default strategy for this class */";      
   print_strategy(global_best);
   print "   /* End of automatically generated code */";
}
